/* Copyright (C) 1993 Free Software Foundation, Inc.
This file is part mkisofs - a iso9660 cdrom filesystem premastering program

The GNU C Library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public License as
published by the Free Software Foundation; either version 2 of the
License, or (at your option) any later version.

The GNU C Library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with the GNU C Library; see the file COPYING.LIB.  If
not, write to the Free Software Foundation, Inc., 675 Mass Ave,
Cambridge, MA 02139, USA.  */

/* Written in April, 1993 by Eric Youngdale */

#include <stdio.h>
#include <dirent.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/dir.h>
#include "iso9660.h"

struct directory_entry{
  struct directory_entry * next;
  struct iso_directory_record isorec;
  unsigned int starting_block;
  unsigned int size;
  unsigned int priority;
  char * name;
  char * table;
  struct directory * filedir;
  struct directory_entry * parent_rec;
  unsigned int flags;
  ino_t inode;  /* Used in the hash table */
  dev_t dev;  /* Used in the hash table */
  char * rr_attributes;
  unsigned int rr_attr_size;
};

struct file_hash{
  struct file_hash * next;
  ino_t inode;  /* Used in the hash table */
  dev_t dev;  /* Used in the hash table */
  unsigned int starting_block;
  unsigned int size;
};
  
struct directory{
  struct directory * next;  /* Next directory at same level as this one */
  struct directory * subdir; /* First subdirectory in this directory */
  struct directory * parent;
  struct directory_entry * contents;
  char * whole_name;  /* Entire path */
  char * name;  /* Entire path */
  unsigned int depth;
  unsigned int size;
  unsigned int extent;
  unsigned short path_index;
};

struct deferred{
  struct deferred * next;
  unsigned int starting_block;
  char * name;
  struct directory * filedir;
  unsigned int flags;
};

extern void sort_n_finish(struct directory *);
extern int goof;
extern struct directory * root;
extern struct directory * reloc_dir;
extern unsigned int next_extent;
extern unsigned int last_extent;
extern unsigned int path_table_size;
extern unsigned int path_table[4];
extern unsigned int path_blocks;
extern char * path_table_l;
extern char * path_table_m;
struct iso_directory_record root_record;
extern int use_RockRidge;
extern int verbose;
extern int all_files;
extern int generate_tables;

extern void scan_directory_tree(char * path, struct directory * parent);
extern void dump_tree(struct directory * node);
extern void assign_directory_addresses(struct directory * root);

extern int iso9660_file_length(const char* name, 
			       struct directory_entry * sresult, int flag);
extern int iso_write(FILE * outfile, char * volid);
extern void generate_path_tables();
extern void generate_iso9660_directories(struct directory *, FILE*);
extern void generate_one_directory(struct directory *, FILE*);
extern void generate_root_record();
extern int iso9660_date(char *, time_t);
extern void add_hash(struct directory_entry *);
extern struct file_hash * find_hash(dev_t, ino_t);
extern void flush_file_hash();
extern int delete_file_hash(struct directory_entry *);
extern struct directory_entry * find_file_hash(char *);
extern void add_file_hash(struct directory_entry *);
extern void finish_cl_pl_entries();

extern void set_723(char *, unsigned int);
extern void set_733(char *, unsigned int);
extern void sort_directory(struct directory_entry **);
extern int generate_rock_ridge_attributes(char *, char *,
					  struct directory_entry *, 
					  struct stat *, struct stat *,
					  int  deep_flag);
extern char * generate_rr_extension_record(char * id,  char  * descriptor,
				    char * source, int  * size);

extern char * extension_record;
extern int extension_record_extent;

/* These are a few goodies that can be specified on the command line, and  are
   filled into the root record */

extern char * preparer;
extern char * publisher;
extern char * copyright;
extern char * biblio;
extern char * abstract;

#define SECTOR_SIZE (2048)
#define ROUND_UP(X)    ((X + (SECTOR_SIZE - 1)) & ~(SECTOR_SIZE - 1))

#define NEED_RE 1
#define NEED_PL  2
#define NEED_CL 4
#define NEED_CE 8
#define NEED_SP 16
